package org.artifactory.core.api.ph.vcs.configuration;

import lombok.Getter;
import org.artifactory.core.api.ph.vcs.configuration.providers.PhPhVcsArtifactoryProviderConfiguration;
import org.artifactory.core.api.ph.vcs.configuration.providers.PhVcsBitbucketProviderConfiguration;
import org.artifactory.core.api.ph.vcs.configuration.providers.PhVcsCustomProviderConfiguration;
import org.artifactory.core.api.ph.vcs.configuration.providers.PhVcsGitProviderConfiguration;
import org.artifactory.core.api.ph.vcs.configuration.providers.PhVcsOldStashProviderConfiguration;
import org.artifactory.core.api.ph.vcs.configuration.providers.PhVcsStashProviderConfiguration;

import java.util.Map;

/**
 * Possible Vcs Git Providers for a remote repository configuration
 */
@Getter
public enum PhVcsGitProvider {
    GITHUB("github", new PhVcsGitProviderConfiguration()),

    BITBUCKET("bitbucket", new PhVcsBitbucketProviderConfiguration()),

    STASH("stash", new PhVcsStashProviderConfiguration()),

    OLDSTASH("oldstash", new PhVcsOldStashProviderConfiguration()),

    ARTIFACTORY("artifactory", new PhPhVcsArtifactoryProviderConfiguration()),

    CUSTOM("custom", new PhVcsCustomProviderConfiguration());

    PhVcsGitProvider(String type, PhVcsProviderConfiguration configuration) {
        this.type = type;
        this.configuration = configuration;
    }

    private final String type;

    private final PhVcsProviderConfiguration configuration;

    public String getPrettyText() {
        return configuration.getPrettyText();
    }

    public String getRefsPrefix() {
        return configuration.getRefsPrefix();
    }

    public String getRepositoryDownloadUrl() {
        return configuration.getRepositoryDownloadUrl();
    }

    public String getResourceDownloadUrl() {
        return configuration.getResourceDownloadUrl();
    }

    public String getReleaseDownloadUrl() {
        return configuration.getReleaseDownloadUrl();
    }

    public Map<String, String> getHeaders() {
        return configuration.getHeaders();
    }
    
}
