package org.artifactory.ui.rest.service.artifacts.browse.treebrowser.tabs.builds;

import org.artifactory.addon.build.BuildAddon;
import org.artifactory.addon.build.artifacts.UsedBy;
import org.artifactory.storage.db.build.service.BuildRunImpl;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.Test;

import javax.ws.rs.core.StreamingOutput;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.stream.Stream;

public class GetArtifactBuildsServiceTest {

    @Test
    public void testGetArtifactBuildsAsStream() {
        Calendar instance1 = Calendar.getInstance();
        instance1.set(2020, Calendar.JANUARY, 1);
        BuildAddon buildAddon = Mockito.mock(BuildAddon.class);
        Date time = instance1.getTime();
        Mockito.when(buildAddon.retrieveDependantBuilds(Mockito.anyString(), Mockito.anyString()))
                .thenReturn(Stream.of(new UsedBy(new BuildRunImpl("n1", "1", time), "m1", "")));

        Mockito.when(buildAddon.retrieveProducedBuilds(Mockito.anyString(), Mockito.anyString()))
                .thenReturn(Stream.of());

        GetArtifactBuildsService getArtifactBuildsService = new GetArtifactBuildsService(null, null);
        StreamingOutput artifactBuildsAsStream = getArtifactBuildsService
                .getArtifactBuildsAsStream(buildAddon, "AB8D78FB3A249B5C4502D14FAB339C06F2E94376",
                        "E343E34D83FF6930F2848CA498D0494C");
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try {
            artifactBuildsAsStream.write(byteArrayOutputStream);
            Assert.assertEquals(byteArrayOutputStream.toString(), "{\"usedBy\":[{\"name\":\"n1\",\"number\":\"1\"," +
                    "\"moduleID\":\"m1\",\"scope\":\"\",\"started\":\"" + time.getTime() + "\"}],\"producedBy\":[]}");

        } catch (IOException e) {
            Assert.fail();
        }

    }
}